@@ -50,6 +50,7 @@ a 32 bits hexadecimal number (e.g. '1234ABCD' without any dash separator).
      - name: partition name
 	   partlabel: partition label
 	   fs: filesystem
+	   fslabel: filesystem label
 	   start: offset
 	   end: offset
 	   features: list of filesystem features
@@ -81,6 +82,12 @@ Optional properties:
 - partlabel -- label for the partition in the GPT partition table. Defaults
 to the `name` property of the partition. May only be used for GPT partitions.
 
+- fslabel -- label for the filesystem. Defaults
+to the `name` property of the partition. The filesystem label can be up to 11
+characters long for {v}fat{12|16|32}, 16 characters long for ext2/3/4, 255
+characters long for btrfs, 512 characters long for hfs/hfsplus and 12 characters
+long for xfs.
+
 - parttype -- set the partition type in the partition table. The string should
 be in a hexadecimal format (2-characters) for msdos partition tables and GUID format
 (36-characters) for GPT partition tables. For instance, "82" for msdos sets the
@@ -167,13 +174,14 @@ import (
 	"github.com/docker/go-units"
 	"github.com/go-debos/fakemachine"
 	"github.com/google/uuid"
-	"gopkg.in/freddierice/go-losetup.v1"
+	"github.com/freddierice/go-losetup/v2"
 	"log"
 	"os"
 	"os/exec"
 	"path"
 	"path/filepath"
 	"sort"
+	"strconv"
 	"strings"
 	"syscall"
 	"time"
@@ -186,6 +194,7 @@ type Partition struct {
 	number          int
 	Name            string
 	PartLabel       string
+	FSLabel         string
 	PartType        string
 	PartUUID        string
 	Start           string
@@ -270,8 +279,17 @@ func (i *ImagePartitionAction) generateFSTab(context *debos.DebosContext) error
 				fs_passno = 2
 			}
 		}
+
+		fsType := m.part.FS
+		switch m.part.FS {
+			case "fat", "fat12", "fat16", "fat32", "msdos":
+				fsType = "vfat"
+			default:
+				break
+		}
+
 		context.ImageFSTab.WriteString(fmt.Sprintf("UUID=%s\t%s\t%s\t%s\t0\t%d\n",
-			m.part.FSUUID, m.Mountpoint, m.part.FS,
+			m.part.FSUUID, m.Mountpoint, fsType,
 			strings.Join(options, ","), fs_passno))
 	}
 
@@ -343,7 +361,7 @@ func (i ImagePartitionAction) formatPartition(p *Partition, context debos.DebosC
 	cmdline := []string{}
 	switch p.FS {
 	case "fat", "fat12", "fat16", "fat32", "msdos", "vfat":
-		cmdline = append(cmdline, "mkfs.vfat", "-n", p.Name)
+		cmdline = append(cmdline, "mkfs.vfat", "-n", p.FSLabel)
 
 		switch p.FS {
 		case "fat12":
@@ -362,7 +380,7 @@ func (i ImagePartitionAction) formatPartition(p *Partition, context debos.DebosC
 		}
 	case "btrfs":
 		// Force formatting to prevent failure in case if partition was formatted already
-		cmdline = append(cmdline, "mkfs.btrfs", "-L", p.Name, "-f")
+		cmdline = append(cmdline, "mkfs.btrfs", "-L", p.FSLabel, "-f")
 		if len(p.Features) > 0 {
 			cmdline = append(cmdline, "-O", strings.Join(p.Features, ","))
 		}
@@ -370,26 +388,26 @@ func (i ImagePartitionAction) formatPartition(p *Partition, context debos.DebosC
 			cmdline = append(cmdline, "-U", p.FSUUID)
 		}
 	case "f2fs":
-		cmdline = append(cmdline, "mkfs.f2fs", "-l", p.Name)
+		cmdline = append(cmdline, "mkfs.f2fs", "-l", p.FSLabel)
 		if len(p.Features) > 0 {
 			cmdline = append(cmdline, "-O", strings.Join(p.Features, ","))
 		}
 	case "hfs":
-		cmdline = append(cmdline, "mkfs.hfs", "-h", "-v", p.Name)
+		cmdline = append(cmdline, "mkfs.hfs", "-h", "-v", p.FSLabel)
 	case "hfsplus":
-		cmdline = append(cmdline, "mkfs.hfsplus", "-v", p.Name)
+		cmdline = append(cmdline, "mkfs.hfsplus", "-v", p.FSLabel)
 	case "hfsx":
-		cmdline = append(cmdline, "mkfs.hfsplus", "-s", "-v", p.Name)
+		cmdline = append(cmdline, "mkfs.hfsplus", "-s", "-v", p.FSLabel)
 		// hfsx is case-insensitive hfs+, should be treated as "normal" hfs+ from now on
 		p.FS = "hfsplus"
 	case "xfs":
-		cmdline = append(cmdline, "mkfs.xfs", "-L", p.Name)
+		cmdline = append(cmdline, "mkfs.xfs", "-L", p.FSLabel)
 		if len(p.FSUUID) > 0 {
 			cmdline = append(cmdline, "-m", "uuid="+p.FSUUID)
 		}
 	case "none":
 	default:
-		cmdline = append(cmdline, fmt.Sprintf("mkfs.%s", p.FS), "-L", p.Name)
+		cmdline = append(cmdline, fmt.Sprintf("mkfs.%s", p.FS), "-L", p.FSLabel)
 		if len(p.Features) > 0 {
 			cmdline = append(cmdline, "-O", strings.Join(p.Features, ","))
 		}
@@ -450,10 +468,31 @@ func (i *ImagePartitionAction) PreNoMachine(context *debos.DebosContext) error {
 
 	img.Close()
 
-	i.loopDev, err = losetup.Attach(imagePath, 0, false)
-	if err != nil {
-		return fmt.Errorf("Failed to setup loop device")
+	// losetup.Attach() can fail due to concurrent attaches in other processes
+	retries := 60
+	for t := 1; t <= retries; t++ {
+		i.loopDev, err = losetup.Attach(imagePath, 0, false)
+		if err == nil {
+			break
+		}
+		log.Printf("Setup loop device: try %d/%d failed: %v", t, retries, err)
+		time.Sleep(200 * time.Millisecond)
 	}
+
+	if err != nil {
+		return fmt.Errorf("Failed to setup loop device: %v", err)
+	}
+
+	// go-losetup doesn't provide a way to change the loop device sector size
+	// see https://github.com/freddierice/go-losetup/pull/10
+	if context.SectorSize != 512 {
+		command := []string{"losetup", "--sector-size", strconv.Itoa(context.SectorSize), i.loopDev.Path()}
+		err = debos.Command{}.Run("losetup", command...)
+		if err != nil {
+			return err
+		}
+	}
+
 	context.Image = i.loopDev.Path()
 	i.usingLoop = true
 
@@ -602,7 +641,14 @@ func (i ImagePartitionAction) Run(context *debos.DebosContext) error {
 		dev := i.getPartitionDevice(m.part.number, *context)
 		mntpath := path.Join(context.ImageMntDir, m.Mountpoint)
 		os.MkdirAll(mntpath, 0755)
-		err = syscall.Mount(dev, mntpath, m.part.FS, 0, "")
+		fsType := m.part.FS
+		switch m.part.FS {
+			case "fat", "fat12", "fat16", "fat32", "msdos":
+				fsType = "vfat"
+			default:
+				break
+		}
+		err = syscall.Mount(dev, mntpath, fsType, 0, "")
 		if err != nil {
 			return fmt.Errorf("%s mount failed: %v", m.part.Name, err)
 		}
@@ -749,6 +795,7 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 
 	num := 1
 	for idx, _ := range i.Partitions {
+		var maxLength int = 0
 		p := &i.Partitions[idx]
 		p.number = num
 		num++
@@ -819,6 +866,32 @@ func (i *ImagePartitionAction) Verify(context *debos.DebosContext) error {
 		if p.FS == "" {
 			return fmt.Errorf("Partition %s missing fs type", p.Name)
 		}
+
+		if p.FSLabel == "" {
+			p.FSLabel = p.Name
+		}
+
+		switch p.FS {
+			case "fat", "fat12", "fat16", "fat32", "msdos", "vfat":
+				maxLength = 11
+			case "ext2", "ext3", "ext4":
+				maxLength = 16
+			case "btrfs":
+				maxLength = 255
+			case "f2fs":
+				maxLength = 512
+			case "hfs", "hfsplus":
+				maxLength = 255
+			case "xfs":
+				maxLength = 12
+			case "none":
+			default:
+				log.Printf("Warning: setting a fs label for %s is unsupported", p.FS)
+		}
+
+		if maxLength > 0 && len(p.FSLabel) > maxLength {
+		        return fmt.Errorf("fs label for %s '%s' is too long", p.Name, p.FSLabel)
+		}
 	}
 
 	for idx, _ := range i.Mountpoints {
